;Input:		VarFuncs = alias where to extract routines to
;		Also initializes variable function jump table
;Output:	64-bit routines extracted to VarFuncs
;		(Requires < 400 bytes of mem)
;			IncVar
;			DecVar
;			SLVar
;			RLVar
;			SRVar
;			RRVar
;			ZeroVar
;			ChZeroVar
;			CPVar
;			SubVar
;			AddVar
;			ExVar
;			LengthVar

SetUp64:
 ld hl,&VarOps
 ld de,VarFuncs
 ld b,11
SU64_Loop:
 push bc
 ld b,0
 ld a,(hl)
 inc hl
 or a
 jr z,SU64_LDMainOps
 ld c,a
 ldir
SU64_LDMainOps:
 ld c,(hl)
 inc hl
 push de
 ldir
 ld c,(hl)
 inc hl
 ex (sp),hl
 ldir
ret_op	=	$C9
 ld a,ret_op
 ld (de),a
 inc de
 pop hl
 pop bc
 djnz SU64_Loop

 ld b,2
jp_op	=	$C3
 ld a,jp_op
SU64_SetJPTab:
 ld (de),a
 inc de
 inc de
 inc de
 djnz SU64_SetJPTab
 ret
;48 bytes


VarOps:

IncVar	=	VarFuncs
;23 bytes -> 6
;20 cc/loop
;Min: 22 cc
;Max: 161 cc
;Most probable: 22 cc
IncVarOps:
.db 0
.db 3
 inc (hl)\ ret nz\ inc l
.db 19

DecVar	=	IncVar+0+3+19+1
;32 bytes -> 9
;27 cc/loop
;Min: 36 cc
;Max: 217 cc
;Most probable: 36 cc
DecVarOps:
.db 2
 ld a,$FF
.db 4
 dec (hl)\ cp (hl)\ ret nz\ inc l
.db 25

SLVar	=	DecVar+2+4+25+1
RLVar	=	SLVar+1
;25 bytes -> 7
;19 cc/loop
;158 (+4) cc total 
SLVarOps:
.db 1
 or a
.db 3
 rl (hl)\ inc l
.db 20

SRVar	=	SLVar+1+3+20+1
RRVar	=	SRVar+1
;29 bytes -> 11
;19 cc/loop
;179 (+4) cc total
;(if macro used, RRVar = 154 cc total)
SRVarOps:
.db 5
 or a\ ld de,7\ add hl,de
.db 3
 rr (hl)\ dec l
.db 20

ZeroVar	=	SRVar+5+3+20+1
;14 bytes -> 6
;11 cc/loop
;98 cc total
ZeroVarOps:
.db 1
 xor a
.db 2
 ld (hl),a\ inc l
.db 13

ChZeroVar	=	ZeroVar+1+2+13+1
;24 bytes -> 7
;16 cc/loop
;133 cc total
ChZeroVarOps:
.db 1
 xor a
.db 3
 or (hl)\ ret nz\ inc l
.db 19

CPVar	=	ChZeroVar+1+3+19+1
;44 bytes -> 14
;27 cc/loop
;Min: 60 cc
;Max: 249 cc
;Most probable: 60 cc (unless equal, then 249 cc)
;(if macro used, Min = 24 cc, Max = 213 cc)
CPVarOps:
.db 6
 ld bc,7\ add hl,bc\ ex de,hl\ add hl,bc
.db 5
 ld a,(de)\ cp (hl)\ ret nz\ dec e\ dec l
.db 32

SubVar	=	CPVar+6+5+32+1
SbcVar	=	SubVar+1
;40 bytes -> 9
;22 cc/loop
;168 (+4) cc total
SubVarOps:
.db 1
 or a
.db 5
 ld a,(de)\ sbc a,(hl)\ ld (de),a\ inc e\ inc l
.db 33

AddVar	=	SubVar+1+5+33+1
AdcVar	=	AddVar+1
;40 bytes -> 9
;29 cc/loop
;224 (+4) cc total
AddVarOps:
.db 1
 or a
.db 5
 ld a,(de)\ adc a,(hl)\ ld (de),a\ inc e\ inc l
.db 33

ExVar		=	AddVar+1+5+33+1
;48 bytes -> 9
;38 cc/loop
;300 cc total
ExVarOps:
.db 0
.db 6
 ld a,(de)\ ldi\ dec l\ ld (hl),a\ inc l
.db 41

LengthVar	=	ExVar+0+6+41+1
;38 bytes -> 13
;17 cc/loop
;Min: 47 cc
;Max: 171 cc
LengthVarOps:
.db 6
 ld de,7\ add hl,de\ inc e\ xor a
.db 4
 or (hl)\ ret nz\ dec e\ dec l
.db 27

;Total compression for all routines:
;357 bytes -> 100

VarJumpTab	=	LengthVar+6+4+27+1
VarFuncJP1	=	VarJumpTab
VarFuncJP2	=	VarFuncJP1+3
