#include "usgard.h"
#include "cmplx64.h"
#include "macro64.h"

;Sets up memory locations of various variables, each 8 bytes long
;This allows for up to about 17 variables in the text memory alone!
VarStart	=	((TEXT_MEM+7)/8)*8
RootVar	=	0*8+VarStart
RemVar	=	1*8+VarStart
ProdVar	=	2*8+VarStart
ExpoVar	=	3*8+VarStart
TempVar	=	4*8+VarStart
TempVar2	=	5*8+VarStart
TempVar3	=	6*8+VarStart
TempVar4	=	7*8+VarStart
VarA	=	8*8+VarStart
VarB	=	9*8+VarStart
VarK	=	10*8+VarStart
VarJ	=	11*8+VarStart
VarN	=	12*8+VarStart

BCDBuffer	=	13*8+VarStart
BCDBuffEnd	=	BCDBuffer+20
SaveHL	=	BCDBuffEnd+1
SaveDE	=	SaveHL+2

VarFuncs	=	GRAPH_MEM

.org 0
.db "PrimeASM85",0

 call CLEARLCD
 call &SetUp64		;decompresses simple 64-bit routines into gfx mem
 call &InitJumpTab	;set up jump table for calls used in macros
 ld de,CURSOR_ROW
 xor a
 ld (de),a
 ld hl,&TitleString
 ld b,6
DispTitle:			;display title
 ld (CURSOR_COL),a
 call D_ZT_STR
 ex de,hl
 inc (hl)
 ex de,hl
 djnz DispTitle
 call &Pause		;wait for key press

 call CLEARLCD
 ld hl,$0000
 ld (CURSOR_ROW),hl
 ld hl,&EnterString
 call D_ZT_STR
 ld hl,$0001
 ld (CURSOR_ROW),hl
 INPUT(VarN)		;gets user input for VarN

 ZERO(VarK)			;VarK = 0
 ROOT(VarN)			;RootVar = (sqrt) VarN
 LD_VAR(VarJ, RootVar)	;VarJ = (sqrt) VarN

 ld b,51			;first iteration goes through 51 elements
 ld ix,&NTILDE+1	;where to start at list
GruntLoop:
 ld a,(ix)			;get element
 push bc
 LD_VAR(VarB, VarK)	;VarB = VarK
 ADD8_A(VarB)		;VarB = VarK + A
 LD_VAR(VarA, VarN)	;VarA = VarN
 DIV(VarA, VarB)	;VarA = VarN / (VarK + A), RemVar = remainder
 CHZERO(RemVar)		;check if remainder = 0
 pop bc
 jr z,Composite		;if rem = 0, found a factor (maybe)
 inc ix				;else, next element
 djnz GruntLoop

 call GET_KEY
 cp K_EXIT
 ret z				;exit program if exit pressed

 ld hl,$0002
 ld (CURSOR_ROW),hl
 LD_VAR(VarA, VarK)	;VarA = VarK
 ld hl,VarB
 push hl
 ld (hl),$10
 inc l
 ld (hl),$27
 call ZeroVar+4
 pop hl				;HL -> VarB = 10000
 ld de,VarA			;DE -> VarA = VarK
 call MultiplyVar64R	;ProdVar = VarK * 10000
 DIV(ProdVar, VarJ)	;ProdVar = (VarK * 10000) / VarJ
 DISP(ProdVar)		;display progress

 ADD8(VarK, 210)	;VarK = VarK + 210
 CP(VarK, VarJ)		;VarK - VarJ
 ld b,52
 ld ix,&NTILDE
 jp c,&GruntLoop	;if VarJ > VarK, repeat
Prime:				;VarN is prime...
 ld hl,$0003
 ld (CURSOR_ROW),hl
 ld hl,&PrimeString
 call D_ZT_STR
 jr Pause
Composite:			;VarN *might* be composite
 DEC(VarA)			;VarA = VarN / [VarK + (ix)] - 1
 CHZERO(VarA)		;if VarA = 0, then VarN = factor...
 jr z,Prime			;...so VarN is prime
 ld hl,$0003
 ld (CURSOR_ROW),hl
 ld hl,&CompositeString
 call D_ZT_STR
 ld de,$0004
 ld (CURSOR_ROW),de
 call D_ZT_STR
 ld a,(ix)
 LD_A(VarA)
 ADD(VarA, VarK)	;VarA = smallest factor of VarN
 DISP(VarA)			;diplay factor
Pause:
 halt
 call GET_KEY
 or a
 jr z,Pause
 ret


#include "SetUp64.h"
#include "jptab.h"
#include "root64.h"
;#include "expo64.h"
#include "mult64.h"
#include "mult8.h"
#include "div64.h"
#include "div8.h"
#include "misc64.h"


NTILDE:
.db   1,  2,  3,  5,  7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59
.db  61, 67, 71, 73, 79, 83, 89, 97,101,103,107,109,113,121,127,131,137,139
.db 143,149,151,157,163,167,169,173,179,181,187,191,193,197,199,209

TitleString:
;	 012345678901234567890
.db "PrimeASM v.9 by JL",0
.db " <JayEll64@aol.com>",0
.db "Original BASIC",0
.db " program by S.T.L.",0
.db "  <STL137@aol.com>",0
.db "Thanks TBL,JMO,ABK",0

EnterString:
.db "Enter a number:",0
PrimeString:
.db "Prime",0
CompositeString:
.db "Composite",0
FactorString:
.db "Factor:",0


.end

